/* HPCU Bi-Weekly Challenge: OpenCL Info
 * Author: Aaron Weeden, Shodor, March 2015
 */
#include <stdio.h>
#include <OpenCL/opencl.h>

#define MAX_STR_LEN 80 /* maximum length of a string */
#define MAX_N_PLATFORMS 4 /* maximum number of OpenCL platforms */
#define MAX_N_DEVICES 4 /* maximum number of devices per platform */

int main() {
  /* number of OpenCL platforms */
  cl_uint platformCount;

  /* IDs of each OpenCL platform */
  cl_platform_id platformIds[MAX_N_PLATFORMS];

  /* number of devices for each platform */
  cl_uint deviceCounts[MAX_N_PLATFORMS];

  /* device IDs for each platform */
  cl_device_id deviceIds[MAX_N_PLATFORMS][MAX_N_DEVICES];

  /* temporary variables to hold platform and device info */
  char infoStr[MAX_STR_LEN];
  cl_bool infoBool;

  /* loop control variables */
  int i;
  int j;

  /* get platform IDs and platform count and print platform count */
  clGetPlatformIDs(MAX_N_PLATFORMS, platformIds, &platformCount);
  printf("Platform Count: %d\n", platformCount);

  /* for each platform, */
  for (i = 0; i < platformCount; i++) {
    /* print platform ID */
    printf("\n_Platform %d_\n", i);

    /* get and print platform profile string */
    clGetPlatformInfo(platformIds[i], CL_PLATFORM_PROFILE, MAX_STR_LEN, infoStr,
      NULL);
    printf("Platform Profile: %s\n", infoStr);

    /* get and print platform version string */
    clGetPlatformInfo(platformIds[i], CL_PLATFORM_VERSION, MAX_STR_LEN, infoStr,
      NULL);
    printf("Platform Version: %s\n", infoStr);

    /* get and print platform name string */
    clGetPlatformInfo(platformIds[i], CL_PLATFORM_NAME, MAX_STR_LEN, infoStr,
      NULL);
    printf("Platform Name: %s\n", infoStr);

    /* get and print platform vendor string */
    clGetPlatformInfo(platformIds[i], CL_PLATFORM_VENDOR, MAX_STR_LEN, infoStr,
      NULL);
    printf("Platform Vendor: %s\n", infoStr);

    /* get device IDs and device count and print device count */
    clGetDeviceIDs(platformIds[i], CL_DEVICE_TYPE_ALL, MAX_N_DEVICES,
      deviceIds[i], &deviceCounts[i]);
    printf("Device Count: %d\n", deviceCounts[i]);

    /* for each device, */
    for (j = 0; j < deviceCounts[i]; j++) {
      /* print device ID */
      printf("\n\t_Device %d_\n", j);

      /* get and print device profile string */
      clGetDeviceInfo(deviceIds[i][j], CL_DEVICE_PROFILE, MAX_STR_LEN, infoStr,
        NULL);
      printf("\tDevice Profile: %s\n", infoStr);

      /* get and print device version string */
      clGetDeviceInfo(deviceIds[i][j], CL_DEVICE_VERSION, MAX_STR_LEN, infoStr,
        NULL);
      printf("\tDevice Version: %s\n", infoStr);

      /* get and print device name */
      clGetDeviceInfo(deviceIds[i][j], CL_DEVICE_NAME, MAX_STR_LEN, infoStr,
        NULL);
      printf("\tDevice Name: %s\n", infoStr);

      /* get and print device vendor */
      clGetDeviceInfo(deviceIds[i][j], CL_DEVICE_VENDOR, MAX_STR_LEN, infoStr,
        NULL);
      printf("\tDevice Vendor: %s\n", infoStr);

      /* get and print whether device is available */
      clGetDeviceInfo(deviceIds[i][j], CL_DEVICE_AVAILABLE, sizeof(cl_bool),
        &infoBool, NULL);
      printf("\tDevice is ");
      if (infoBool == CL_FALSE) {
        printf("NOT ");
      }
      printf("available.\n");

      /* get and print whether device compiler is available */
      clGetDeviceInfo(deviceIds[i][j], CL_DEVICE_COMPILER_AVAILABLE,
        sizeof(cl_bool), &infoBool, NULL);
      printf("\tDevice compiler is ");
      if (infoBool == CL_TRUE) {
        printf("available.\n");
      }
      else {
        printf("unavaiable.\n");
      }
    } /* end for each device */
  } /* end for each platform */

  return 0;
}
